# Copyright 2017 Matthew Treinish
#
# This file is part of pymochad
#
# pymochad is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# pymochad is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with pymochad.  If not, see <http://www.gnu.org/licenses/>.

import mock

from pymochad import controller
from pymochad.tests import base


class TestController(base.TestCase):

    @mock.patch('socket.socket')
    def test_status(self, socket_mock):
        cont = controller.PyMochad()
        with mock.patch.object(cont, 'read_data') as read_mock:
            with mock.patch.object(cont, 'send_cmd') as send_mock:
                cont.status()
                send_mock.assert_called_once_with('st\n')
                read_mock.assert_called_once_with()



# Copyright 2016 Matthew Treinish
#
# This file is part of pymochad
#
# pymochad is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# pymochad is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with pymochad.  If not, see <http://www.gnu.org/licenses/>.


class Device(object):
    """PyMochad Device class

    This class represents an X1 device connected to your controller

    :param PyMochad controller: A PyMochad controller object for the device to
                                use
    :param str address: The device address
    :param str comm_type: The communication type to use for the device. This is
                          either pl (for power line) or rf (for radio
                          frequency)
    """
    def __init__(self, controller, address, comm_type='pl'):
        self.controller = controller
        self.address = address
        if comm_type not in ['pl', 'rf']:
            msg = 'Mochad only supports a comm_type of pl or rf'
            raise TypeError(msg)
        self.comm_type = comm_type

    def send_cmd(self, cmd):
        """Send a raw command to device.

        :param str cmd: The command to send to the device
        """
        cmd_str = ' '.join([self.comm_type, self.address, cmd])
        self.controller.send_cmd(cmd_str + '\n')

    def get_status(self):
        """Get the on/off status for the devices

        :returns: Device status
        :rtype: str
        """
        cmd_str = 'getstatus ' + self.address + '\n'
        self.controller.send_cmd(cmd_str)
        return self.controller.read_data().lower()

    def get_statussec(self):
        """Get the on/off status for the X10 Security devices

        :returns: Device status
        :rtype: str
        """
        cmd_str = 'getstatussec ' + self.address + '\n'
        self.controller.send_cmd(cmd_str)
        return self.controller.read_data().lower()



# Copyright 2016 Matthew Treinish
#
# This file is part of pymochad
#
# pymochad is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# pycmus is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with pymochad. If not, see <http://www.gnu.org/licenses/>.


class MochadException(Exception):
    message = "An unknown exception occurred"

    def __init__(self, *args, **kwargs):
        super(MochadException, self).__init__()
        try:
            self._error_string = self.message % kwargs
        except Exception:
            # at least get the core message out if something happened
            self._error_string = self.message
        if len(args) > 0:
            # If there is a non-kwarg parameter, assume it's the error
            # message or reason description and tack it on to the end
            # of the exception message
            # Convert all arguments into their string representations...
            args = ["%s" % arg for arg in args]
            self._error_string = (self._error_string +
                                  "\nDetails: %s" % '\n'.join(args))

    def __str__(self):
        return self._error_string


class ConfigurationError(MochadException):
    message = "A configuration error occured"



# Copyright 2016 Matthew Treinish
#
# This file is part of pymochad
#
# pymochad is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# pymochad is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with pymochad. If not, see <http://www.gnu.org/licenses/>.

import logging
import socket
import time

import six

from pymochad import exceptions

LOG = logging.getLogger(__name__)


class PyMochad(object):
    """PyMochad controller class

    This class is used to create a PyMochad controller object that is used to
    send commands to a running PyMochad daemon.

    :param str server: The host to connect to the pymochad socket on, it
                       defaults to localhost
    :param int port: The port to use for remote connections. If one is not
                     provided it will just use the default port of 1099.
    """
    def __init__(self, server=None, port=1099):
        super(PyMochad, self).__init__()
        self.port = port
        self.server = server or 'localhost'
        self._connect()

    def _connect(self):
        for addr in socket.getaddrinfo(self.server, self.port):
            af, socktype, proto, cannonname, sa = addr
            try:
                self.socket = socket.socket(af, socktype, proto)
                self.socket.connect(sa)
            except Exception:
                continue
            break
        else:
            raise exceptions.ConfigurationError(
                "Unable to connect to server %s" % self.server)
        self.socket.setblocking(0)

    def reconnect(self):
        """Reconnect when mochad server is restarted/lost connection."""
        if self.socket:
            self.socket.close()
        self._connect()

    def send_cmd(self, cmd):
        """Send a raw command to mochad.

        :param str cmd: The command to send to mochad
        """
        self.socket.sendall(six.binary_type(cmd.encode('utf8')))

    def read_data(self):
        """Read data from mochad

        :return data: The data returned over the mochad socket
        :rtype: str
        """

        total_data = []
        while True:
            try:
                data = self.socket.recv(4096)
                if not data:
                    break
                total_data.append(six.text_type(data.decode('utf8')))
                line_break = six.binary_type('\n'.encode('utf8'))
                if data.endswith(line_break):
                    break
            except socket.error as e:
                if e.errno == socket.errno.EWOULDBLOCK:
                    time.sleep(1)
                    continue
                else:
                    raise
        return ''.join(total_data)

    def status(self):
        """Send a show device status command.

        :return status: The status of device including RF security devices
        :rtype: str
        """
        self.send_cmd('st\n')
        return self.read_data()
